#!/usr/bin/env python

# -*- coding: utf-8 -*-

## Copyright (C)2008 Jean-Philippe Camguilhem <jpc@makina-corpus.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file LICENSE. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

"""Build Configuration Documentation for Plone's Product or Zope3 packages.

    :Author: Jean-Philippe Camguilhem <jpc@makina-corpus.com> 
"""

import sys
import os
import shutil

try:
    import libxml2
except:
    print 'This script depends on libxml2 please install it !'
    sys.exit()

try:
    import libxslt
except:
    print 'This script depends on libxslt please install it !'
    sys.exit()

XSLSHEET = 'doc.xsl'
DOCDIR = 'docs'
STATICSDIR = '_statics'
FILES_XML = os.path.join(DOCDIR, 'files.xml')
PROFILE_HTML = os.path.join(DOCDIR, 'profiles.html')
CONF_HTML = os.path.join(DOCDIR, 'configure.html')
PROFILE_DIR = 'profiles'

ZEXT = '.zcml'
PEXT = '.xml'
DIRTAG = '  <directory dirpath="%s">\n%s  </directory>\n'
FILETAG = '    <filename filename="%s" path="%s"/>\n'

def checkDocDir(root, docdir=DOCDIR, staticsdir=STATICSDIR):
    """check if docs an docs/_statics/files exist. If not creates them.
    """
    if not docdir in os.listdir('.'):
        os.mkdir(docdir)

    if not staticsdir in os.listdir(docdir):
        os.mkdir('%s/%s' % (docdir, staticsdir))

    statics = os.listdir(os.path.join(root, staticsdir))
    # copy _statics/ files if not exist
    for static in statics:
        if not os.path.isfile(os.path.join(sources, docdir, staticsdir, static)
                              ) and not static.startswith('.'):

            shutil.copy(os.path.join(root, staticsdir, static),
                        os.path.join(os.path.join(docdir, staticsdir, static)))

def buildXMLFilesList(ext=ZEXT, profiledir='.'):
    """Return directories and filenames tags.
    """
    xml = []
    for root, dirs, files in os.walk(profiledir):
        filestoadd = [filename for filename in files if filename.endswith(ext)]
        if filestoadd:
            xml.append(DIRTAG % (root, ''.join([FILETAG % (filename, root)
                                                for filename in filestoadd])))

    return ''.join(xml)

def writeTmpXML(ext, profiledir, filesxml=FILES_XML):
    """Build files.xml with for each directory files and pathnames.
    """
    try:
        xml = open(filesxml,"w")
        xml.write('<?xml version="1.0"?>')
        xml.write('<root>\n')
        xml.write(buildXMLFilesList(ext, profiledir))
        xml.write('</root>')

    finally:
        xml.close()

def transformXMLtoDocHTML(html, xsl=XSLSHEET):
    """Apply doc.xsl on files.xml to generate html .
    """
    styledoc = libxml2.parseFile(xsl)
    style = libxslt.parseStylesheetDoc(styledoc)
    doc = libxml2.parseFile(FILES_XML)
    result = style.applyStylesheet(doc, None)
    style.saveResultToFilename(html, result, 0)
    style.freeStylesheet()
    doc.freeDoc()
    result.freeDoc()

def createHtmlDocFile(ext, html, profiledir='.'):
    """."""
    writeTmpXML(ext=ext, profiledir=profiledir)
    transformXMLtoDocHTML(html=html)
    os.remove(FILES_XML)

def makeDoc(sources):
    """Called from __main__"""
    root = os.getcwd()

    # copy XSLSHEET for relative paths
    shutil.copy(XSLSHEET, os.path.join(sources, XSLSHEET))

    # go to sources directory and create docs/_statics/files if needed
    os.chdir(sources)
    checkDocDir(root)

    # build configure.html
    createHtmlDocFile(ext=ZEXT, html=CONF_HTML)

    # test if has profiles directory :Plone Product Generic Setup case"
    if PROFILE_DIR in os.listdir(os.getcwd()):
        createHtmlDocFile(ext=PEXT, html=PROFILE_HTML, profiledir=PROFILE_DIR)

    os.remove(XSLSHEET)
    os.chdir(root)

if __name__ == "__main__":
    usage = 'No package specified.\nUsage $ %s package/path/' % os.sys.argv[0]

    if len(sys.argv) < 2:
        print usage
        sys.exit()

    else:
        sources = sys.argv[1]
        if os.path.isdir(sources):
            makeDoc(sources)

        else:
            print '%s is not a directory !\n%s' % (sources, usage)
            sys.exit()

